'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (gulp, plugins, config) {
  const browserSync = plugins.browserSync;
  const bsParent = config.browserSync;
  const sources = config.sources;
  const utils = config.utils;
  const test = config.test;
  const parentMiddleware = bsParent.middleware;
  const parentOpenFn = bsParent.open;
  const internalHost = sources.internalHost;
  const devPort = sources.devPort;
  const buildDir = sources.buildDir;
  const addbase = utils.addbase;
  const logError = utils.logError;

  const baseOpen = `http://${ internalHost }:${ devPort }`;
  const callReturn = _boilerUtils2.default.callAndReturn;
  const callParent = _boilerUtils2.default.runParentFn;
  const runFn = _boilerUtils2.default.runCustomTask;

  let openPath = baseOpen;
  let shouldOpen = true;

  return cb => {
    if (_lodash2.default.isFunction(parentOpenFn)) {
      openPath = callReturn(config)(parentOpenFn, baseOpen);
    } else if (_lodash2.default.isBoolean(parentOpenFn)) {
      shouldOpen = parentOpenFn;
    }

    const expireHeaders = (req, res, next) => {
      res.setHeader('cache-control', 'public, max-age=0');
      next();
    };

    const defaultMiddleware = [(req, res, next) => {
      res.setHeader('Access-Control-Allow-Origin', '*');
      res.setHeader('Access-Control-Allow-Methods', 'POST, GET, OPTIONS');
      res.setHeader('Access-Control-Allow-Headers', 'authorization, accept');
      res.setHeader('Access-Control-Max-Age', '1728000');
      if (req.method === 'OPTIONS') {
        res.end();
      } else {
        next();
      }
    }, expireHeaders];

    const middleware = _lodash2.default.isFunction(parentMiddleware) ? parentMiddleware(config, defaultMiddleware) : defaultMiddleware;

    if (_lodash2.default.isUndefined(middleware)) {
      logError({
        err: new Error('You must `return` a middleware Function or Array'),
        plugin: '[browser-sync: middleware]'
      });
    }

    const bsConfig = {
      open: false,
      port: devPort,
      server: {
        baseDir: addbase(buildDir),
        middleware
      },
      online: false
    };

    const parentConfig = callParent(arguments, {
      data: bsConfig
    });

    var _parentConfig$data = parentConfig.data;
    const data = _parentConfig$data === undefined ? {} : _parentConfig$data;
    const fn = parentConfig.fn;


    if (test) {
      Object.assign(data, { ghostMode: false });
    }

    const task = done => {
      browserSync(data, () => {
        if (shouldOpen) {
          (0, _open2.default)(openPath);
        }

        done();
      });
    };

    return runFn(task, fn, cb);
  };
};

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _open = require('open');

var _open2 = _interopRequireDefault(_open);

var _boilerUtils = require('boiler-utils');

var _boilerUtils2 = _interopRequireDefault(_boilerUtils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = exports['default'];